﻿using UnityEngine;
using HarmonyLib;
using System.Collections.Generic;
using UnityEngine.UI;
using HMLLibrary;
using UnityEngine.SceneManagement;

public class AlteredIslandGen : Mod
{
    Harmony harmony;
    static Button unloadButton;
    static Button.ButtonClickedEvent eventStore = null;
    static bool started;
    static bool CanUnload
    {
        get { return eventStore == null; }
        set
        {
            if (!value && eventStore == null)
            {
                eventStore = unloadButton.onClick;
                unloadButton.onClick = new Button.ButtonClickedEvent();
                unloadButton.onClick.AddListener(delegate { Debug.LogError("Mod must be unloaded on the main menu"); });
                modInstance.modlistEntry.jsonmodinfo.isModPermanent = true;
                ModManagerPage.RefreshModState(modInstance.modlistEntry);
            }
            else if (value && eventStore != null)
            {
                unloadButton.onClick = eventStore;
                eventStore = null;
                modInstance.modlistEntry.jsonmodinfo.isModPermanent = false;
                ModManagerPage.RefreshModState(modInstance.modlistEntry);
            }
        }
    }
    public void Start()
    {
        started = false;
        unloadButton = modlistEntry.modinfo.unloadBtn.GetComponent<Button>();
        if (SceneManager.GetActiveScene().name != Semih_Network.MenuSceneName)
        {
            unloadButton.onClick.Invoke();
            throw new ModLoadException("Mod must be loaded on the main menu");
        }
        started = true;
        harmony = new Harmony("com.aidanamite.AlteredIslandGen");
        harmony.PatchAll();
        Log("Mod has been loaded!");
    }

    public void OnModUnload()
    {
        if (!started)
            return;
        harmony.UnpatchAll(harmony.Id);
        Log("Mod has been unloaded!");
    }

    public override void WorldEvent_WorldLoaded()
    {
        CanUnload = false;
    }

    public override void WorldEvent_WorldUnloaded()
    {
        CanUnload = true;
    }
}

class ModLoadException : System.Exception
{
    public ModLoadException(string message) : base(message) { }
}

[HarmonyPatch(typeof(Landmark), "Initialize")]
public class Patch_IslandSpawn
{
    static GameObject titaniumModel = null;
    static GameObject TitaniumModel
    {
        get
        {
            if (titaniumModel == null)
                foreach (var p in Resources.FindObjectsOfTypeAll<MeshRenderer>())
                    if (p.name == "Raw_Titanium")
                    {
                        titaniumModel = p.gameObject;
                        break;
                    }
            return titaniumModel;
        }
    }
    static List<GameObject> plasticModels = new List<GameObject>();
    static List<GameObject> PlasticModels
    {
        get
        {
            if (plasticModels.Count > 0 && plasticModels[0] == null)
                plasticModels.Clear();
            if (plasticModels.Count == 0)
                foreach (var p in Resources.FindObjectsOfTypeAll<MeshRenderer>())
                    if (p.name.StartsWith("Pickup_Floating_Plastic") && !p.name.EndsWith("(Clone)"))
                        plasticModels.Add(p.gameObject);
            return plasticModels;
        }
    }
    static SO_ItemYield titaniumYield;
    static SO_ItemYield plasticYield;
    static void Prefix(ref Landmark __instance, bool ___initialized)
    {
        if (!___initialized)
        {
            bool flagT = TitaniumModel;
            if (!flagT)
                Debug.LogError("Could not find titanium ore model");
            bool flagP = PlasticModels.Count > 0;
            if (!flagP)
                Debug.LogError("Could not find plastic model");
            if (!flagT && !flagP)
                return;
            var rand = new System.Random(System.BitConverter.ToInt32(System.BitConverter.GetBytes(__instance.uniqueLandmarkIndex), 0));
            foreach (var item in __instance.GetComponentsInChildren<LandmarkItem_PickupItem>())
                if (flagT && (item.name.StartsWith("Pickup_Landmark_Iron") || item.name.StartsWith("Pickup_Landmark_Copper")) && rand.NextDouble() < 0.05)
                    ModifyPickup(__instance, item, "Titanium", TitaniumModel, ref titaniumYield, ItemManager.GetItemByName("TitaniumOre"));
                else if (flagP && item.name.StartsWith("Pickup_Landmark_Scrap") && item.name.Contains("OceanBottom") && rand.NextDouble() < 0.2)
                    ModifyPickup(__instance, item, "Plastic", PlasticModels[(int)(PlasticModels.Count * rand.NextDouble())], ref plasticYield, ItemManager.GetItemByName("Plastic"));
        }
    }
    public static void ModifyPickup(Landmark island, LandmarkItem_PickupItem item, string name, GameObject newModel, ref SO_ItemYield yieldAsset, Item_Base loot)
    {
        var pickup = item.GetComponent<PickupItem>();
        if (yieldAsset == null)
        {
            yieldAsset = new SO_ItemYield();
            yieldAsset.name = "ItemYield_Pickup_Landmark_" + name;
            yieldAsset.yieldAssets = new List<Cost>()
            {
                new Cost(loot, 1)
            };
        }
        pickup.yieldHandler.yieldAsset = yieldAsset;
        pickup.yieldHandler.ResetYield();
        pickup.specificPickups = new Pickup_Specific[0];
        pickup.itemInstance = new ItemInstance();
        pickup.dropper = null;
        item.name = "Pickup_Landmark_" + name;
        var model = item.transform.Find("Model");
        model.GetComponent<MeshFilter>().mesh = newModel.GetComponent<MeshFilter>().mesh;
        model.GetComponent<MeshRenderer>().material = newModel.GetComponent<MeshRenderer>().material;
        var collider = item.GetComponent<BoxCollider>();
        var modelCollider = newModel.GetComponent<BoxCollider>();
        if (!modelCollider)
            modelCollider = newModel.GetComponentInChildren<BoxCollider>();
        if (modelCollider)
        {
            collider.size = modelCollider.size;
            collider.center = modelCollider.center;
        }
        var parent = island.transform.Find(name);
        if (parent == null)
        {
            parent = new GameObject(name).transform;
            parent.SetParent(island.transform, false);
        }
        item.transform.SetParent(parent, true);
    }
}